#include "chord.h"

#include <netinet/in.h>
#include <sys/uio.h>
#include <stdint.h>

#define CHORD_FIND 0
#define GET_PREDECESSOR 1
#define NOTIFY 2
#define GET_SUCCESSOR 3
#define CHECK 4
#define PRED_EXIT 5
#define SUCC_EXIT 6
#define OOB 7

struct chord_network {
	size_t hash_size;
	pthread_rwlock_t *finger_table_lock;
	struct chord_node *finger_table;

	int num_successors;
	pthread_rwlock_t successor_list_lock;
	struct chord_node *successor_list;

	pthread_rwlock_t local_node_lock;
	struct chord_node local_node;

	unsigned char pred_failed;
	pthread_rwlock_t predecessor_lock;
	struct chord_node predecessor;

	int ssocketfd;

	pthread_t server_thread, maint_thread;

	int (*handler)(int socketfd, void *arg);
	void *arg;
};

/* chord_common.c */

/* memory cleanup functions */
void chord_free(void *arg);
void chord_mpz_clear(void *arg);

/* lock cleanup functions */
void chord_mutex_unlock(void *arg);
void chord_rwlock_unlock(void *arg);

/* chord node manipulation functions */
int chord_nodecmp(const struct chord_node *node1,

		const struct chord_node *node2, size_t hash_size);
void chord_nodecpy(struct chord_node *dest, const struct chord_node *src,
		size_t hash_size);

/* networking functions */
int chord_read(int fd, void *buf, size_t count);
int chord_write(int fd, void *buf, size_t count);
int chord_readv(int fd, struct iovec *iov, int iovcnt);
int chord_writev(int fd, struct iovec *iov, int iovcnt);
void chord_close(void *arg);
int chord_connect(const struct chord_node *node);
int chord_check(struct chord_node *node);

/* node search functions */
unsigned char find_next_node(struct chord_network *network,
		const unsigned char *hash, struct chord_node *next_node);
int chord_next_node(int socketfd, const unsigned char *hash, size_t hash_size,
		struct chord_node *next_node, unsigned char *last_node);

/* error handling functions */
void successor_failed(struct chord_network *network);

/* hash manipulation functions */
void hash_address(struct chord_network *network);
unsigned char hash_inrange(unsigned char *hash, unsigned char *bot_hash,
		unsigned char *top_hash, size_t hash_size);

/* chord_maint.c */
void *chord_maintainance(void *arg);

/* chord_server.c */
void *chord_serve_request(void *arg);
