#include "galois.h"

#include <vector>
#include <functional>
#include <algorithm>

namespace netcodec {
	template<unsigned size> std::vector<typename galois<size>::type> encode(
			const std::vector<typename galois<size>::type> &coefficients,
			const std::vector<std::vector<typename galois<size>::type>> &data)
	{
		std::vector<typename galois<size>::type> result(data[0].size());
		for (auto i = result.begin(); i != result.end(); i++) {
			galois<size> temp = { 0 };
			auto coefficient = coefficients.begin();
			for (auto packet = data.begin(); packet != data.end(); packet++, coefficient++)
				temp += galois<size>(*coefficient) * galois<size>((*packet)[i - result.begin()]);

			*i = temp;
		}

		return result;
	}

	template std::vector<uint8_t> encode<1>(
			const std::vector<uint8_t> &coefficients,
			const std::vector<std::vector<uint8_t>> &data);

	template std::vector<uint16_t> encode<2>(
			const std::vector<uint16_t> &coefficients,
			const std::vector<std::vector<uint16_t>> &data);


	template<unsigned size> unsigned decode(
			const std::vector<typename galois<size>::type> &coefficients,
			const std::vector<typename galois<size>::type> &packet,
			std::vector<std::vector<galois<size>>> &data, unsigned rank)
	{
		typedef typename galois<size>::type type;

		std::copy(coefficients.begin(), coefficients.end(), data[rank].begin());
		std::copy(packet.begin(), packet.end(), data[rank].begin() + coefficients.size());

		auto not_zero = std::bind2nd(std::not_equal_to<galois<size>>(), 0);
		unsigned row;
		for (row = 0; row != rank; row++) {
			auto dst = std::find_if(data[rank].begin(),
					data[rank].begin() + coefficients.size(), not_zero);

			unsigned dst_order = dst - data[rank].begin();
			if (dst_order == coefficients.size())
				return rank;

			auto src = std::find_if(data[row].begin(),
					data[row].begin() + coefficients.size(), not_zero);

			unsigned src_order = src - data[row].begin();
			if (src_order < dst_order)
				continue;

			if (src_order > dst_order)
				break;

			galois<size> scale = data[rank][dst_order] / data[row][src_order];
			for ( ; dst != data[rank].end(); dst++, src++)
				*dst -= scale * *src;
		}
		if (row == rank) {
			if (std::find_if(data[rank].begin(), data[rank].begin() + coefficients.size(),
						not_zero) == data[rank].begin() + coefficients.size())

				return rank;
		} else
			std::rotate(data.begin() + row, data.begin() + rank, data.begin() + rank + 1);

		return rank + 1;
	}

	template unsigned decode<1>(const std::vector<uint8_t> &, const std::vector<uint8_t> &,
			std::vector<std::vector<galois<1>>> &, unsigned);

	template unsigned decode<2>(const std::vector<uint16_t> &, const std::vector<uint16_t> &,
			std::vector<std::vector<galois<2>>> &, unsigned);

	template<unsigned size> void back_substitute(std::vector<std::vector<galois<size>>> &data)
	{
		unsigned dst_row = data.size();
		while (dst_row-- != 0) {
			std::transform(data[dst_row].begin() + dst_row + 1, data[dst_row].end(),
					data[dst_row].begin() + dst_row + 1,
					std::bind2nd(std::divides<galois<size>>(), data[dst_row][dst_row]));

			for (unsigned src_row = dst_row + 1; src_row != data.size(); src_row++) {
				galois<size> scale = data[dst_row][src_row];
				auto dst = data[dst_row].begin() + data.size();
				auto src = data[src_row].begin() + data.size();
				for (; src != data[src_row].end(); src++, dst++)
					*dst -= scale * *src;
			}
		}
	}

	template void back_substitute<1>(std::vector<std::vector<galois<1>>> &);
	template void back_substitute<2>(std::vector<std::vector<galois<2>>> &);
}
