#include "get_file.h"

#include <p2pvdn/socket.h>
#include <p2pvdn/hash.h>

#include <chord.h>

#include <vector>
#include <fstream>

#include <getopt.h>

namespace get_file {
	struct opts {
		std::string filename;
		std::string output;
		std::string node;
		std::string service;

		opts(int argc, char *const argv[]);
	};

	opts::opts(int argc, char *const argv[])
		: node("127.0.0.1"), service("50000")
	{
		const struct option options[] = {
			{ "address", required_argument, NULL, 'a' },
			{ "port", required_argument, NULL, 'p' },
			{ NULL, 0, NULL, 0 }
		};
		const std::string short_opts("a:p:");

		int opt;
		while ((opt = getopt_long(argc, argv, short_opts.c_str(), options, NULL)) != -1)
			switch (opt) {
				case 'a':
					node = optarg;
					break;
				case 'p':
					service = optarg;
					break;
				default:
					throw std::exception();
			}

		if (!*(argv + optind))
			throw std::exception();

		filename = *(argv++ + optind);
		if (*(argv + optind))
			output = *(argv + optind);
	}
}

get_file::ctx::ctx(const opts &opts)
	: hash(opts.filename, 20)
{
	if (opts.output == "-")
		output = &std::cout;
	else {
		if (opts.output.size())
			fstream.open(opts.output);
		else
			fstream.open(opts.filename);

		output = &fstream;
	}

	chord_init_node(opts.node.c_str(), opts.service.c_str(), &gw_node);

	struct chord_node node;
	node.hash = new unsigned char[hash.size()];
	chord_successor_ext(&gw_node, hash.size(), hash.get(), &node);
	p2pvdn::socket socket(chord_oob(&node));
	delete[] node.hash;

	socket << p2pvdn::command::send_peer_list << hash << p2pvdn::command::close;
	socket >> peers;

	node.address = peers[0].first.get_address();
	node.port = peers[0].first.get_port();
	socket = {chord_oob(&node)};

	socket << p2pvdn::command::key_info << hash << p2pvdn::command::close;
	socket >> file_opts;
}

int main(int argc, char *argv[])
{
	get_file::opts opts(argc, argv);
	get_file::ctx ctx(opts);

	//TODO: choose peers?
	if (ctx.file_opts.coeff_size == 1)
		stream::stream<1>(ctx);
	else
		stream::stream<2>(ctx);

	return EXIT_SUCCESS;
}
