#ifndef HASH_H
#define HASH_H

#include <vector>
#include <iostream>

#include <cstring>
#include <cstdint>

namespace p2pvdn {
	//TODO: inherit vector<unsigned char>
	class hash {
		std::vector<unsigned char> key;

		public:
		hash(const unsigned char *key, unsigned size)
			: key(key, key + size)
		{
		}

		hash(const std::vector<unsigned char> &key)
			:key(key)
		{
		}

		hash(std::vector<unsigned char> &&key)
			:key(std::move(key))
		{
		}

		hash(const p2pvdn::socket &socket)
		{
			unsigned size;
			socket >> size;
			key = std::vector<unsigned char>(size);
			socket.recvall(&key[0], key.size());
		}

		hash(const hash &hash)
			: key(hash.key)
		{
		}

		hash(hash &&hash)
			: key(std::move(hash.key))
		{
		}

		hash(const std::string &path, unsigned size);

		hash &operator=(const hash &key) throw(std::bad_alloc)
		{
			*this = hash(key);

			return *this;
		}

		hash &operator=(hash &&hash) throw()
		{
			key = std::move(hash.key);

			return *this;
		}

		bool operator<(const hash &) const throw();

		friend const p2pvdn::socket &operator<<(const p2pvdn::socket &, const hash &);

		const unsigned char *get() const
		{
			return &key[0];
		}

		unsigned size() const
		{
			return key.size();
		}

		friend std::ostream &operator<<(std::ostream &stream, const hash &hash);
	};

	inline const p2pvdn::socket &operator<<(const p2pvdn::socket &socket, const hash &hash)
	{
		unsigned size = hash.key.size();
		socket << size;
		socket.sendall(&hash.key[0], size);

		return socket;
	}

	inline const p2pvdn::socket &operator>>(const p2pvdn::socket &socket, hash &key)
	{
		key = { socket };

		return socket;
	}

	inline std::ostream &operator<<(std::ostream &stream, const hash &hash)
	{
		stream << std::hex;
		for (auto digit = hash.key.begin(); digit != hash.key.end(); digit++)
			stream << static_cast<unsigned>(*digit);

		stream << std::dec;

		return stream;
	}
}

#endif
