#ifndef PACKET_H
#define PACKET_H

#include <vector>

namespace p2pvdn {
	template<unsigned coefficient_size> struct packet_type;

	template<> struct packet_type<1> {
		typedef uint8_t value;
	};

	template<> struct packet_type<2> {
		typedef uint16_t value;
	};

	template<unsigned coefficient_size> class packet {
		typedef typename packet_type<coefficient_size>::value type;
		std::vector<type> coefficients;
		std::vector<type> data;

		public:
		packet()
		{
		}

		packet(const std::vector<type> &coefficients, const std::vector<type> &data)
			: coefficients(coefficients), data(data)
		{
		}

		packet(std::vector<type> &&coefficients, const std::vector<type>data)
			: coefficients(std::move(coefficients)), data(data)
		{
		}

		packet(const std::vector<type> &coefficients, std::vector<type> &&data)
			: coefficients(coefficients), data(std::move(data))
		{
		}

		packet(std::vector<type> &&coefficients, std::vector<type> &&data)
			: coefficients(std::move(coefficients)), data(std::move(data))
		{
		}

		packet(const socket &socket, const file_opts &opts)
			: coefficients(opts.num_slices * opts.coeff_size / coefficient_size),
			data(opts.packet_elements() * opts.coeff_size / coefficient_size)
		{
			socket.recvall(&coefficients[0], coefficients.size() * coefficient_size);
			socket.recvall(&data[0], data.size() * coefficient_size);
		}

		packet(const packet &packet)
			: coefficients(packet.coefficients), data(packet.data)
		{
		}

		packet(packet &&packet)
			: coefficients(std::move(packet.coefficients)),
			data(std::move(packet.data))
		{
		}

		packet &operator=(const packet &packet)
		{
			coefficients = packet.coefficients;
			data = packet.data;

			return *this;
		}

		packet &operator=(packet &&packet)
		{
			coefficients = std::move(packet.coefficients);
			data = std::move(packet.data);

			return *this;
		}

		operator type *() const
		{
			type *buffer = new type[coefficients.size() + data.size()];
			copy(coefficients.begin(), coefficients.end(), buffer);
			copy(data.begin(), data.end(), buffer + coefficients.size());

			return buffer;
		}

		template<unsigned size> friend inline const socket &
			operator<<(const socket &, const packet<size> &);

		unsigned size() const
		{
			return data.size();
		}

		unsigned header_size() const
		{
			return coefficients.size();
		}
	};

	template<unsigned coefficient_size> inline const socket &
		operator<<(const socket &socket, const packet<coefficient_size> &packet)
	{
		socket.sendall(&packet.coefficients[0], packet.coefficients.size() * coefficient_size);
		socket.sendall(&packet.data[0], packet.data.size() * coefficient_size);

		return socket;
	}
}

#endif
