#ifndef FILE_DATA_H
#define FILE_DATA_H

#include <boost/thread/shared_mutex.hpp>

#include <p2pvdn/socket.h>
#include <p2pvdn/file_opts.h>
#include <p2pvdn/packet.h>

#include <memory>
#include <vector>
#include <list>
#include <thread>

namespace vdnd {
	class chunk {
		mutable boost::shared_mutex mutex;
		std::list<std::shared_ptr<p2pvdn::packet<1>>> packets;

		public:
		chunk()=default;

		chunk(const chunk &chunk)
		{
			boost::shared_lock<boost::shared_mutex> lock{chunk.mutex};
			packets = chunk.packets;
		}

		chunk(chunk &&chunk)
			: packets(std::move(chunk.packets))
		{
		}

		std::vector<std::shared_ptr<p2pvdn::packet<1>>> get_packets() const
		{
			boost::shared_lock<boost::shared_mutex> lock{mutex};

			return std::vector<std::shared_ptr<p2pvdn::packet<1>>>(packets.begin(), packets.end());
		}

		void add_packet(const p2pvdn::packet<1> &packet)
		{
			std::lock_guard<boost::shared_mutex> lock{mutex};
			packets.emplace_back(new p2pvdn::packet<1>(packet));
		}

		void add_packet(const p2pvdn::packet<1> &&packet)
		{
			std::lock_guard<boost::shared_mutex> lock{mutex};
			packets.emplace_back(new p2pvdn::packet<1>(std::move(packet)));
		}
	};

	class file_data {
		const p2pvdn::file_opts file_opts;
		std::vector<chunk> chunks;

		public:
		file_data(const p2pvdn::file_opts &file_opts)
			: file_opts(file_opts), chunks(file_opts.num_chunks())
		{
		}

		const p2pvdn::file_opts &get_file_opts() const
		{
			return file_opts;
		}

		chunk &get_chunk(unsigned chunk_num)
		{
			return chunks.at(chunk_num);
		}

		const chunk &get_chunk(unsigned chunk_num) const
		{
			return chunks.at(chunk_num);
		}

		unsigned num_chunks() const
		{
			return chunks.size();
		}

		friend const p2pvdn::socket &operator>>(const p2pvdn::socket &, file_data &);
	};

	const p2pvdn::socket &operator>>(const p2pvdn::socket &, file_data &);
}

#endif
